@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( App::MatrixTool );
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use MIME::Base64 qw( decode_base64 );
 use Protocol::Matrix qw( verify_json_signature );
@@ -190,18 +190,18 @@ sub run
       $self->output( "$result{version} keys from $result{server_name}:" );
       $self->output();
 
-      my $store = $self->key_store;
-      my %cached_keys = $store->get_keys( server_name => $result{server_name} );
+      my $store = $self->server_key_store;
+      my %cached_keys = $store->list( server => $result{server_name} );
 
       foreach ( @{ $result{keys} } ) {
          $self->output( "Key id $_->{id}" );
          $self->output( "  " . $self->format_binary( $_->{key} ) );
 
          if( !exists $cached_keys{ $_->{id} } ) {
-            $store->put_key(
-               server_name => $result{server_name},
-               key_id      => $_->{id},
-               public_key  => $_->{key},
+            $store->put(
+               server => $result{server_name},
+               id     => $_->{id},
+               data   => $_->{key},
             ) unless $opts->{no_store};
          }
          elsif( $cached_keys{ $_->{id} } eq $_->{key} ) {
