@@ -8,14 +8,15 @@ package App::MatrixTool::HTTPClient;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use Future 0.33; # ->catch
 use Future::Utils qw( repeat_until_success );
 use IO::Async::Loop;
 use IO::Async::Resolver 0.68; # failure details
 use IO::Async::Resolver::DNS 0.06 qw( ERR_NO_HOST ); # Future return with failure details
-use JSON qw( decode_json );
+use JSON qw( encode_json decode_json );
+use URI;
 
 use Socket qw( getnameinfo NI_NUMERICHOST SOCK_RAW );
 
@@ -143,30 +144,62 @@ sub request
    my $self = shift;
    my %params = @_;
 
+   my $uri = URI->new;
+   $uri->path( $params{path} );
+   $uri->query_form( %{ $params{params} } ) if $params{params};
+
    my $ua  = $self->ua;
-   my $req = $params{request} // HTTP::Request->new( $params{method} => $params{path},
+   my $req = $params{request} // HTTP::Request->new( $params{method} => $uri,
       [ Host => $params{server} ],
    );
    $req->protocol( "HTTP/1.1" );
 
+   if( defined $params{content} and ref $params{content} ) {
+      $req->content( encode_json( delete $params{content} ) );
+      $req->header( Content_length => length $req->content );
+      $req->header( Content_type => "application/json" );
+   }
+
    if( $self->{print_request} ) {
       print STDERR "Sending HTTP request to $params{server}\n";
       print STDERR "  $_\n" for split m/\n/, $req->as_string( "\n" );
    }
 
-   $self->resolve_matrix( $params{server} )->then( sub {
+   my $path = $req->uri->path;
+
+   # Different kinds of request need resolving either as a client or as a
+   # federated server
+   my $resolve_f;
+   if( $path =~ m{^/_matrix/key/} ) {
+      $resolve_f = $self->resolve_matrix( $params{server} )->then( sub {
+         my @res = @_;
+         Future->done( map {
+            { SSL => 1, host => $_->{target}, port => $_->{port}, family => $self->{family} }
+         } @res );
+      });
+   }
+   elsif( $path =~ m{^/_matrix/client/} ) {
+      my ( $server, $port ) = $params{server} =~ m/^([^:]+)(?::(\d+))?$/ or
+         die "Unable to parse server '$params{server}'\n";
+      $resolve_f = Future->done(
+         { SSL => 1, port => $port // 443, host => $server, family => $self->{family} }
+      );
+   }
+   else {
+      die "Unsure how to resolve server for path $path\n";
+   }
+
+   $resolve_f->then( sub {
       my @res = @_;
 
       repeat_until_success {
          my $res = shift;
-         print STDERR "Using target $res->{target} port $res->{port}\n" if $self->{print_request};
+         print STDERR "Using target $res->{host} port $res->{port}\n" if $self->{print_request};
 
          $ua->do_request(
             %params,
+            %$res,
             request => $req,
-            SSL     => 1,
-            host    => $res->{target},
-            port    => $res->{port},
          )->on_done( sub {
             my ( $response ) = @_;
             if( $self->{print_response} ) {
